@@ -1,7 +1,7 @@
 list(rd_family_title = list(
   common = "Other commonly-used functions: ",
-  reading = "Other functions for reading in CIFTI or GIFTI metric data: ",
-  writing = "Other functions for writing CIFTI or GIFTI metric data: ",
+  reading = "Other functions for reading in CIFTI or GIFTI data: ",
+  writing = "Other functions for writing CIFTI or GIFTI data: ",
   surfing = "Other functions for working with GIFTI surface geometry data: ",
   manipulating = "Other functions for manipulating `xifti` objects: "
 ))
\ No newline at end of file
